@@ -32,13 +32,14 @@
 #include <string.h>
 #include <time.h>
 #include <math.h>
-#include <dirent.h>
 #include <gnome.h>
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <gconf/gconf-client.h>
 #include "games-frame.h"
+#include "games-files.h"
+#include "games-gridframe.h"
 #include "glines.h"
 
 #define KEY_DIR "/apps/glines"
@@ -48,6 +49,8 @@
 #define KEY_SAVED_SCORE "/apps/glines/saved/score"
 #define KEY_SAVED_FIELD "/apps/glines/saved/field"
 #define KEY_SAVED_PREVIEW "/apps/glines/saved/preview"
+#define KEY_WIDTH "/apps/glines/saved/width"
+#define KEY_HEIGHT "/apps/glines/saved/height"
 
 #define NCOLORS 7
 
@@ -55,20 +58,38 @@ GConfClient *conf_client = NULL;
 
 GtkWidget *draw_area;
 static GtkWidget *app, *appbar, *pref_dialog;
-GtkWidget *next_draw_area; /* XXX Shouldn't be this much externls! */
+GtkWidget *preview_widgets[3];
 field_props field[FIELDSIZE * FIELDSIZE];
-GdkPixbuf *ball_pixbuf = NULL;
-GdkPixmap *surface = NULL;
+
+/* The unscaled pixbuf as read from file. Cached here to save file reads. */
+GdkPixbuf *raw_pixbuf = NULL;
+/* The tile images with balls rendered on them. */
+GdkPixmap *ball_pixmap = NULL;
+/* The balls rendered to a size appropriate for the preview. */
+GdkPixmap *preview_pixmaps[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+/* A pixmap of a blank tile. */
+GdkPixmap *blank_pixmap = NULL;
 
 GtkWidget *fast_moves_toggle_button = NULL;
 
-GList * theme_list = NULL;
+GamesFileList * theme_file_list = NULL;
 
 int active = -1;
 int target = -1;
 int inmove = 0;
 int score = 0;
 
+int boxsize;
+
+/* The width and height of the main window. */
+#define MIN_WIDTH  190
+#define MIN_HEIGHT 240
+int width;
+int height;
+
+int preview_height = 0;
+int preview_width = 0;
+
 int move_timeout = 100;
 int animate_id = 0;
 int preview[3];
@@ -96,23 +117,37 @@ static void
 load_image (gchar *fname,
 	    GdkPixbuf **pixbuf)
 {
-	gchar *tmp, *fn;
+	gchar *tmp, *fn = NULL;
 	GdkPixbuf *image;
-    
-	tmp = g_build_filename ("glines", fname, NULL);
 
+	tmp = g_build_filename ("glines", fname, NULL);
+	
 	fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
 	g_free (tmp);
 
-	if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
-		char *message = g_strdup_printf (_("Glines couldn't find image file:\n%s\n\n"
-						   "Please check your Glines installation"), fn);
-		GtkWidget *w = gtk_message_dialog_new (GTK_WINDOW (app),
-						       GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-						       GTK_MESSAGE_ERROR,
-						       GTK_BUTTONS_OK,
-						       message,
-						       NULL);
+
+	/* Just think of this as an if statement, but we want to use break. */
+	while (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
+		char * message;
+		GtkWidget * w;
+
+		/* ball.png was replaced with balls.svg. */
+		if (g_utf8_collate (fname, "ball.png") == 0) {
+			tmp = g_build_filename ("glines", "balls.svg", NULL);
+			fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
+			g_free (tmp);
+			if (g_file_test (fn, G_FILE_TEST_EXISTS))
+				break;
+		}
+
+		message = g_strdup_printf (_("Glines couldn't find image file:\n%s\n\n"
+					     "Please check your Glines installation"), fn);
+		w = gtk_message_dialog_new (GTK_WINDOW (app),
+					    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    message,
+					    NULL);
 		gtk_dialog_set_has_separator (GTK_DIALOG (w), FALSE);
 		gtk_dialog_run (GTK_DIALOG (w));	
 		g_free (message);
@@ -122,6 +157,7 @@ load_image (gchar *fname,
 	image = gdk_pixbuf_new_from_file (fn, NULL);
 	g_free( fn );
 
+
 	if (*pixbuf)
 		g_object_unref (*pixbuf);
 
@@ -129,13 +165,97 @@ load_image (gchar *fname,
 }
 
 static void
-help_cb (GtkWidget * widget, gpointer data)
+refresh_pixmaps (void)
 {
-#if 0
-  GnomeHelpMenuEntry help_entry = { "glines", "user-guide/usage.html#PREFERENCES-DIALOGBOX" };
+	static GdkGC * grid_gc = NULL;
+	GdkGC * gc;
+        GdkColormap *cmap;
+        GdkColor color;
+	GdkPixbuf *ball_pixbuf;
+	gint i;
+
+	/* Since we get called both by configure and after loading an image.
+	 * it is possible the pixmaps aren't initialised. If they aren't
+	 * we don't do anything. */
+
+	if (!ball_pixmap)
+		return;
+
+	if (!grid_gc) {
+		grid_gc = gdk_gc_new (draw_area->window);
+
+		gdk_color_parse ("#525F6C", &color);
+		cmap = gtk_widget_get_colormap (draw_area);
+		gdk_colormap_alloc_color (cmap, &color, FALSE, TRUE);
+		gdk_gc_set_foreground (grid_gc, &color);
+	}
+
+	ball_pixbuf = gdk_pixbuf_scale_simple (raw_pixbuf, 4*boxsize, 
+					       7*boxsize, 
+					       GDK_INTERP_BILINEAR);
+
+	gc = gdk_gc_new (ball_pixmap);
+	gdk_gc_set_foreground (gc, &backgnd.color);
+
+	gdk_draw_rectangle (ball_pixmap, gc, TRUE, 0, 0, 4*boxsize, 7*boxsize);
+	for (i=0; i<4; i++) 
+		gdk_draw_line (ball_pixmap, grid_gc, i*boxsize, 0, i*boxsize,
+			       7*boxsize);
+	for (i=0; i<7; i++)
+		gdk_draw_line (ball_pixmap, grid_gc, 0, i*boxsize, 4*boxsize,
+			       i*boxsize);
+
+	gdk_draw_pixbuf (ball_pixmap, gc,
+			 ball_pixbuf, 0, 0, 0, 0, boxsize*4, boxsize*7,
+			 GDK_RGB_DITHER_NORMAL, 0, 0);
+	gdk_draw_rectangle (blank_pixmap, gc, TRUE, 0, 0, boxsize, boxsize);
+	gdk_draw_line (blank_pixmap, grid_gc, 0, 0, 0, boxsize);
+	gdk_draw_line (blank_pixmap, grid_gc, 0, 0, boxsize, 0);
+
+	g_object_unref (ball_pixbuf);
+	g_object_unref (gc);
+}
+
+static void
+refresh_preview_pixmaps (void)
+{
+	int i;
+	GdkPixbuf * scaled;
+	GtkWidget * widget = preview_widgets[0];
+
+	/* Like the refresh_pixmaps() function, we may be called before
+	 * the window is ready. */
+	if (preview_height == 0)
+		return;
+
+	/* We create pixmaps for each of the ball colours and then
+	 * set them as the background for each widget in the preview array.
+	 * This code assumes that each preview window is identical. */
+
+	if (preview_pixmaps[0])
+		for (i=0; i<7; i++)
+			g_object_unref (preview_pixmaps[i]);
+
+	scaled = gdk_pixbuf_scale_simple (raw_pixbuf, 4*preview_width, 
+					  7*preview_height, 
+					  GDK_INTERP_BILINEAR);
+
+	for (i=0; i<7; i++) {
+		preview_pixmaps[i] = gdk_pixmap_new (widget->window,
+						     preview_width, 
+						     preview_height, -1);
+		gdk_draw_rectangle (preview_pixmaps[i], 
+				    widget->style->bg_gc[GTK_STATE_NORMAL],
+				    TRUE, 0, 0, preview_width, preview_height);
+
+		gdk_draw_pixbuf (preview_pixmaps[i], widget->style->white_gc, 
+				 scaled, 0, i*preview_height, 0, 0, 
+				 preview_width, preview_height,
+				 GDK_RGB_DITHER_NORMAL, 0, 0);
+	}
+
+	g_object_unref (scaled);
 
-  gnome_help_display (NULL, &help_entry);
-#endif
 }
 
 static void
@@ -223,7 +343,14 @@ init_preview (void)
 void
 draw_preview (void)
 {
-	gtk_widget_queue_draw (next_draw_area);
+	int i;
+
+	for (i=0; i<3; i++) {
+		gdk_window_set_back_pixmap (preview_widgets[i]->window,
+					    preview_pixmaps[preview[i]-1],
+					    FALSE);
+		gdk_window_clear (preview_widgets[i]->window);
+	}
 }
 
 static void
@@ -301,15 +428,8 @@ init_new_balls (int num, int prev)
 void
 draw_box (GtkWidget *widget, int x, int y)
 {
-	gtk_widget_queue_draw_area (widget, x * BOXSIZE, y * BOXSIZE,
-				    BOXSIZE, BOXSIZE);
-}
-
-void
-draw_ball (GtkWidget *widget, int x, int y)
-{
-	gtk_widget_queue_draw_area (widget, x * BOXSIZE, y * BOXSIZE,
-				    BOXSIZE, BOXSIZE);
+	gtk_widget_queue_draw_area (widget, x * boxsize, y * boxsize,
+				    boxsize, boxsize);
 }
 
 static int
@@ -385,7 +505,7 @@ deactivate (GtkWidget *widget, int x, int y)
 {
 	field[active].active = 0;
 	field[active].phase = 0; 
-	draw_ball(widget, x, y);
+	draw_box (widget, x, y);
 	active = -1;
 }
 
@@ -399,8 +519,8 @@ button_press_event (GtkWidget *widget, GdkEvent *event)
 
 	if(inmove) return TRUE;
 	gtk_widget_get_pointer (widget, &x, &y);
-	fx = x / BOXSIZE;
-	fy = y / BOXSIZE;
+	fx = x / boxsize;
+	fy = y / boxsize;
         gnome_appbar_set_status(GNOME_APPBAR(appbar), "");
 	if(field[fx + fy*9].color == 0)
 	{
@@ -459,108 +579,53 @@ button_press_event (GtkWidget *widget, GdkEvent *event)
 	return TRUE;
 }
 
-/* Redraw the whole preview */
-static gint
-preview_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer gp)
-{
-	GdkGC *gc;
-	gint i;
-
-        gc = gdk_gc_new (widget->window);
-	for (i = 0; i < 3; i++) {
-		/* Draw the ball */
-		gdk_draw_pixbuf (widget->window, gc, ball_pixbuf,
-				 0, (preview[i] - 1) * BALLSIZE,
-				 5 + i * BOXSIZE, 5, BALLSIZE, BALLSIZE,
-				 GDK_RGB_DITHER_NORMAL, 0, 0);
-	}
-        g_object_unref (gc);
-
-	return FALSE;
-}
-
-static void
-draw_grid (void)
-{
-        GdkColormap *cmap;
-        GdkGC *gc;
-        GdkColor color;
-        gint x, y;
-
-        if (!gdk_color_parse ("#525F6C", &color)) {
-                return;
-        }
-
-        cmap = gtk_widget_get_colormap (draw_area);
-
-        gc = gdk_gc_new (draw_area->window);
-
-	gdk_colormap_alloc_color (cmap, &color, FALSE, TRUE);
-        gdk_gc_set_foreground (gc, &color);
-
-        for (x = BOXSIZE; x < FIELDSIZE*BOXSIZE; x = x + BOXSIZE) {
-                gdk_draw_line (surface, gc, x, 0, x, FIELDSIZE*BOXSIZE);
-        }
-        for (y = BOXSIZE; y < FIELDSIZE*BOXSIZE; y = y + BOXSIZE) {
-                gdk_draw_line (surface, gc, 0, y, FIELDSIZE*BOXSIZE, y);
-        }
-
-        g_object_unref (gc);
-}
-
-
-
 /* Redraw a part of the field */
 static gint
 field_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer gp)
 {
+	GdkWindow * window = widget->window;
 	GdkGC *gc;
 	guint x_start, x_end, y_start, y_end, i, j, idx;
 
-	x_start = event->area.x / BOXSIZE;
-	x_end = x_start + event->area.width / BOXSIZE;
+	x_start = event->area.x / boxsize;
+	x_end = (event->area.x + event->area.width - 1) / boxsize + 1;
 
-	y_start = event->area.y / BOXSIZE;
-	y_end = y_start + event->area.height / BOXSIZE;
-	
-	draw_grid ();
+	y_start = event->area.y / boxsize;
+	y_end = (event->area.y + event->area.height - 1) / boxsize + 1;
 
         gc = gdk_gc_new (draw_area->window);
 
 	for (i = y_start; i < y_end; i++) {
 		for (j = x_start; j < x_end; j++) {
+			int x, y, phase, color;
+
 			idx = j + i * FIELDSIZE;
+			x = idx % FIELDSIZE;
+			y = idx / FIELDSIZE;
 
-			/* Draw the box */
-			gdk_gc_set_foreground (gc, &backgnd.color);
-			gdk_draw_rectangle (surface, 
-					    gc,
-					    TRUE, j * BOXSIZE+1, i * BOXSIZE+1,
-					    BOXSIZE-1, BOXSIZE-1);
-
-			/* Draw the ball */
 			if (field[idx].color != 0) {
-				int phase = field[idx].phase;
-				int color = field[idx].color;
-				int x = idx % FIELDSIZE;
-				int y = idx / FIELDSIZE;
+				phase = field[idx].phase;
+				color = field[idx].color - 1;
 
 				phase = ABS(ABS(3-phase)-3);
-				/*gc = widget->style->fg_gc[GTK_STATE_NORMAL];*/
 
-				gdk_draw_pixbuf(surface, gc, ball_pixbuf,
-						phase * BALLSIZE, (color - 1) * BALLSIZE,
-						5 + x * BOXSIZE, 5 + y * BOXSIZE,
-						BALLSIZE, BALLSIZE,
-						GDK_RGB_DITHER_NORMAL, 0, 0);
+				gdk_draw_drawable (window, gc, ball_pixmap,
+						   phase * boxsize, 
+						   color * boxsize,
+						   x * boxsize, 
+						   y * boxsize,
+						   boxsize, boxsize);
+			} else {
+				gdk_draw_drawable (window, gc, blank_pixmap,
+						   0, 0,
+						   x * boxsize, 
+						   y * boxsize,
+						   boxsize, boxsize);
 			}
 		}
 	}
         g_object_unref (gc);
 
-	gdk_window_set_back_pixmap (widget->window, surface, 0);
-	gdk_window_clear (widget->window);
-
 	return FALSE;
 }
 
@@ -593,7 +658,7 @@ addscore (int num)
 	
 
 static int
-check_goal (GtkWidget *widget, int num, int is_score)
+check_goal (GtkWidget *widget, int num)
 {
 	int count = 0;
 	int subcount = 0;
@@ -695,14 +760,12 @@ check_goal (GtkWidget *widget, int num, int is_score)
 	
 	if (count >= 4)
 	{
+		char string[20];
+
 		kill_tagged (widget, count);
-		if (is_score)
-		{
-			char string[20];
-			score += addscore (count+1);
-			g_snprintf (string, 19, "%d", score);
-			gtk_label_set_text (GTK_LABEL (scorelabel), string);
-		}
+		score += addscore (count+1);
+		g_snprintf (string, 19, "%d", score);
+		gtk_label_set_text (GTK_LABEL (scorelabel), string);
 
 		subcount = 1;
 	}
@@ -750,16 +813,16 @@ animate (gpointer gp)
 			active = -1;
 			field[newactive].phase = 0;
 			field[newactive].active = 0;
-			draw_ball (widget, x, y);
+			draw_box (widget, x, y);
 			reset_pathsearch ();
-			if (!check_goal (widget, newactive, 1))
+			if (!check_goal (widget, newactive))
 			{
 				for (x = 0; x < 3; x++)
 				{
 					int tmp = init_new_balls (1, x);
-					draw_ball (widget, tmp % FIELDSIZE,
+					draw_box (widget, tmp % FIELDSIZE,
 						   tmp / FIELDSIZE);
-					check_goal (widget, tmp, 0);
+					check_goal (widget, tmp);
 					if (check_gameover () == -1)
 						return FALSE;
 				}
@@ -775,7 +838,7 @@ animate (gpointer gp)
 	field[active].phase++;
 	if(field[active].phase >= 4)
 		field[active].phase = 0;
-	draw_ball (widget, x, y);
+	draw_box (widget, x, y);
 
 	return TRUE;
 }
@@ -907,59 +970,39 @@ bg_color_callback (GtkWidget *widget, gpointer data)
 static void
 load_theme ()
 {
-	load_image (ball_filename, &ball_pixbuf);
+	load_image (ball_filename, &raw_pixbuf);
+	refresh_pixmaps ();
+	refresh_preview_pixmaps ();
 }
 
 static void
 set_selection (GtkWidget *widget, char *data)
 {
-	GList * entry;
+	gchar *entry;
 
-	entry = g_list_nth (theme_list,
+	entry = games_file_list_get_nth (theme_file_list,
 			    gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
 	gconf_client_set_string (conf_client,
 				 KEY_BALL_THEME,
-				 entry->data, NULL);
+				 entry, NULL);
 }
 
-static void
-fill_menu (GtkWidget *menu, char * mtype, gboolean bg)
+static GtkWidget * 
+fill_menu (void)
 {
-	struct dirent *e;
-	DIR *dir;
-	int itemno = 0;
 	gchar *dname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
 						  ("glines"), FALSE, NULL);
 
-	if (theme_list) {
-		g_list_foreach (theme_list, (GFunc) g_free, NULL);
-		g_list_free (theme_list);
-	}
-	theme_list = NULL;
-		
-	dir = opendir (dname);
+	if (theme_file_list)
+		g_object_unref (theme_file_list);
+
+	theme_file_list = games_file_list_new_images (dname, NULL);
 	g_free (dname);
+	games_file_list_transform_basename (theme_file_list);
 
-	if (!dir)
-		return;
-	
-	while ((e = readdir (dir)) != NULL){
-		gchar *s = g_strdup (e->d_name);
-
-		if (! g_strrstr (e->d_name, mtype)) {
-			g_free (s);
-			continue;
-		}
-
-		gtk_combo_box_append_text (GTK_COMBO_BOX (menu), s);
-		theme_list = g_list_append (theme_list, s);
-
-		if (! strcmp (ball_filename, s))
-			gtk_combo_box_set_active (GTK_COMBO_BOX (menu), itemno);
-			  
-		itemno++;
-	}
-	closedir (dir);
+	return games_file_list_create_widget (theme_file_list, ball_filename,
+					      GAMES_FILE_LIST_REMOVE_EXTENSION |
+					      GAMES_FILE_LIST_REPLACE_UNDERSCORES);
 }
 
 static void
@@ -975,16 +1018,7 @@ set_fast_moves_callback (GtkWidget *widget, gpointer *data)
 static void
 pref_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 {
-	switch (response)
-		{
-		case GTK_RESPONSE_HELP:
-			help_cb (NULL, NULL);
-			break;
-		case GTK_RESPONSE_CLOSE:
-		default:
-			gtk_widget_hide (GTK_WIDGET (dialog));
-			break;
-		}      
+	gtk_widget_hide (GTK_WIDGET (dialog));
 }
 
 static void
@@ -1021,10 +1055,9 @@ game_props_callback (GtkWidget *widget, void *data)
 			gtk_misc_set_alignment (GTK_MISC (l), 0, 0.5);
 			gtk_table_attach_defaults (GTK_TABLE (table), l, 0, 1, 0, 1);
 	    
-			omenu = gtk_combo_box_new_text ();
+			omenu = fill_menu ();
 			g_signal_connect (G_OBJECT (omenu), "changed",
 					  G_CALLBACK (set_selection), NULL);
-			fill_menu (omenu, ".png", FALSE);
 			gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
 
 
@@ -1068,6 +1101,15 @@ game_props_callback (GtkWidget *widget, void *data)
 	gtk_window_present (GTK_WINDOW (pref_dialog));
 }
 
+static int
+window_resize_cb (GtkWidget * widget, GdkEventConfigure * event, void * data)
+{
+	gconf_client_set_int (conf_client, KEY_WIDTH, event->width, NULL);
+	gconf_client_set_int (conf_client, KEY_HEIGHT, event->height, NULL);
+
+	return FALSE;
+}
+
 static int
 game_quit_callback (GtkWidget *widget, void *data)
 {
@@ -1077,28 +1119,41 @@ game_quit_callback (GtkWidget *widget, void *data)
 	return FALSE;
 }
 
+static int
+preview_configure_cb (GtkWidget * widget, GdkEventConfigure * event)
+{
+	preview_width = event->width;
+	preview_height = event->height;
+
+	refresh_preview_pixmaps ();
+
+	draw_preview ();
+
+	return TRUE;
+}
+
 static int
 configure_event_callback (GtkWidget *widget, GdkEventConfigure *event)
 {
-	if (surface)
-		{
-			gint old_w, old_h;
-			
-			gdk_drawable_get_size (surface, &old_w, &old_h);
-			if (old_w == event->width && old_h == event->height)
-				return TRUE;
-			g_object_unref (surface);
-		}
+	int xboxsize, yboxsize;
 
-	surface = gdk_pixmap_new (draw_area->window, event->width,
-				  event->height,
-				  gdk_drawable_get_visual
-				  (draw_area->window)->depth);
-	gdk_draw_rectangle (surface, 
-			    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-			    TRUE, 0, 0, -1, -1);
+	if (ball_pixmap) 
+		g_object_unref (ball_pixmap);
+
+	if (blank_pixmap)
+		g_object_unref (blank_pixmap);
+
+	xboxsize = event->width/FIELDSIZE;
+	yboxsize = event->height/FIELDSIZE;
+	boxsize = MIN (xboxsize, yboxsize);
+
+	ball_pixmap = gdk_pixmap_new (draw_area->window, boxsize*4, boxsize*7,
+				      -1);
+	blank_pixmap = gdk_pixmap_new (draw_area->window, boxsize, boxsize, -1);
+	refresh_pixmaps ();
 
 	refresh_screen ();
+
 	return TRUE;
 }
 
@@ -1166,15 +1221,6 @@ load_properties (void)
 	load_theme ();
 }
 
-static gint clamp_int (gint input, gint low, gint high)
-{
-	if (input < low)
-		input = low;
-	if (input > high)
-		input = high;
-	return input;
-}
-
 static void
 restart (void)
 {
@@ -1196,10 +1242,10 @@ restart (void)
 	{
 		for(i = 0; i < FIELDSIZE * FIELDSIZE; i++)
 		{
-			field[i].color = clamp_int (buf[i*4] - 'h', 1, NCOLORS);
-			field[i].pathsearch = clamp_int (buf[i*4 + 1] - 'h', -1, FIELDSIZE*FIELDSIZE);
-			field[i].phase = clamp_int (buf[i*4 + 2] - 'h', 0, 3);
-			field[i].active = clamp_int (buf[i*4 + 3] - 'h', -1, 1);
+			field[i].color = CLAMP (buf[i*4] - 'h', 1, NCOLORS);
+			field[i].pathsearch = CLAMP (buf[i*4 + 1] - 'h', -1, FIELDSIZE*FIELDSIZE);
+			field[i].phase = CLAMP (buf[i*4 + 2] - 'h', 0, 3);
+			field[i].active = CLAMP (buf[i*4 + 3] - 'h', -1, 1);
 		}
 		g_free(buf);
 	}
@@ -1208,7 +1254,7 @@ restart (void)
 	if(buf)
 	{
 		for(i = 0; i < 3; i++)
-			preview[i] = clamp_int (buf[i] - 'h', 1, NCOLORS);
+			preview[i] = CLAMP (buf[i] - 'h', 1, NCOLORS);
 		g_free(buf);
 	}
 }
@@ -1303,7 +1349,7 @@ move_timeout_changed_cb (GConfClient *client,
 
 	timeout_tmp = gconf_client_get_int (client,
 					    KEY_MOVE_TIMEOUT, NULL);
-	timeout_tmp = clamp_int (timeout_tmp, 1, 1000);
+	timeout_tmp = CLAMP (timeout_tmp, 1, 1000);
 	if (timeout_tmp != move_timeout)
 		move_timeout = timeout_tmp;
   
@@ -1311,10 +1357,12 @@ move_timeout_changed_cb (GConfClient *client,
 }
 
 static void
-init_config (void)
+init_config (int argc, char ** argv)
 {
 	conf_client = gconf_client_get_default ();
 
+        gconf_init (argc, argv, NULL);
+
 	gconf_client_add_dir (conf_client,
 			      KEY_DIR,
 			      GCONF_CLIENT_PRELOAD_ONELEVEL,
@@ -1331,14 +1379,26 @@ init_config (void)
 				 KEY_BACKGROUND_COLOR,
 				 bg_color_changed_cb,
 				 NULL, NULL, NULL);
+
+	/* These are here because they are only loaded once. */
+	width = gconf_client_get_int (conf_client,
+				      KEY_WIDTH, NULL);
+	width = MAX (width, MIN_WIDTH);
+       
+	height = gconf_client_get_int (conf_client,
+				       KEY_HEIGHT, NULL);
+	height = MAX (height, MIN_HEIGHT);
 }
 
 int
 main (int argc, char *argv [])
 {
-	GtkWidget *frame;   
-	GtkWidget *vbox, *hbox;
+	GtkWidget *alignment;   
+	GtkWidget *gridframe, *label;
+	GtkWidget *vbox, *table, *hbox;
+	GtkWidget *preview_hbox;
 	GnomeClient *client;
+	int i;
 	
 	gnome_score_init ("glines");
 
@@ -1354,9 +1414,7 @@ main (int argc, char *argv [])
 			    GNOME_PARAM_POPT_TABLE, NULL,
 			    GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
-        gconf_init (argc, argv, NULL);
-        gconf_client_add_dir (gconf_client_get_default (), KEY_DIR,
-                              GCONF_CLIENT_PRELOAD_NONE, NULL);
+        init_config (argc, argv);
 
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/glines.png");
 	client = gnome_master_client ();
@@ -1372,12 +1430,13 @@ main (int argc, char *argv [])
 		reset_game ();
 
 	app = gnome_app_new ("glines", _("Lines"));
+	gtk_window_set_default_size (GTK_WINDOW (app), width, height);
+	gtk_widget_set_size_request (GTK_WIDGET (app), MIN_WIDTH, MIN_HEIGHT);
 
-	gtk_window_set_resizable (GTK_WINDOW (app), FALSE);
 	g_signal_connect (G_OBJECT (app), "delete_event",
 	                  G_CALLBACK (game_quit_callback), NULL);
 	g_signal_connect (G_OBJECT (app), "configure_event",
-			  G_CALLBACK (configure_event_callback), NULL);
+	                  G_CALLBACK (window_resize_cb), NULL);
 
 	appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
 	gnome_app_set_statusbar (GNOME_APP (app), GTK_WIDGET(appbar));  
@@ -1389,58 +1448,67 @@ main (int argc, char *argv [])
 
 	gnome_app_install_menu_hints(GNOME_APP (app), mainmenu);
   
- 	vbox = gtk_vbox_new (FALSE, 0);
-
+	vbox = gtk_vbox_new (FALSE, 0);
 	gnome_app_set_contents (GNOME_APP (app), vbox);
 
+ 	table = gtk_table_new (10, 1, TRUE);
+	gtk_box_pack_start_defaults (GTK_BOX (vbox), table);
+
 	hbox = gtk_hbox_new(FALSE, 0);
-	frame = games_frame_new (_("Next Balls"));
-	gtk_container_set_border_width (GTK_CONTAINER (frame), 0);
-	games_frame_set (GAMES_FRAME (frame), 0);
+	gtk_table_attach_defaults (GTK_TABLE (table), hbox, 0, 1, 0, 1);
+	label = gtk_label_new (g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Next Balls:")));
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
 
-	draw_area = gtk_drawing_area_new ();
-	next_draw_area = gtk_drawing_area_new ();
+	alignment = gtk_alignment_new (0, 0.5, 0.3, 1);
+	gtk_box_pack_start (GTK_BOX (hbox), alignment, TRUE, TRUE, 0);
 
-	gtk_widget_set_size_request (GTK_WIDGET (draw_area),
-				     BOXSIZE * 9, BOXSIZE * 9);
-	gtk_widget_set_size_request (GTK_WIDGET (next_draw_area),
-				     BOXSIZE * 3, BOXSIZE);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (frame), next_draw_area);
-	gtk_frame_set_label_align (GTK_FRAME (frame), 0, 0);
+	gridframe = games_grid_frame_new (3, 1);
+	gtk_container_add (GTK_CONTAINER (alignment), gridframe);
+	
+	preview_hbox = gtk_hbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (gridframe), 
+			   preview_hbox);
 
-	frame = games_frame_new (_("Score"));
-	gtk_container_set_border_width (GTK_CONTAINER (frame), 0);
+	for (i=0; i<3; i++) {
+		preview_widgets[i] = gtk_drawing_area_new ();
+		gtk_box_pack_start (GTK_BOX (preview_hbox),
+				    preview_widgets[i], TRUE, TRUE, 0);
+		/* So we have all the windows at configure time since
+		 * we only hook into one of the configure events. */
+		/* Yes, this is an evil hack. */
+		gtk_widget_realize (preview_widgets[i]);
+	}
+	g_signal_connect (G_OBJECT (preview_widgets[0]), "configure_event",
+			  G_CALLBACK (preview_configure_cb), NULL);
 
 	scorelabel = gtk_label_new ("");
-	gtk_container_add (GTK_CONTAINER (frame), scorelabel);
 
-	gtk_box_pack_end (GTK_BOX (hbox), frame, 0, 0, 0);
+	gtk_box_pack_end (GTK_BOX (hbox), scorelabel, FALSE, FALSE, 5);
 
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), draw_area);
-
-	gtk_widget_set_events (draw_area, gtk_widget_get_events(draw_area) |GDK_BUTTON_PRESS_MASK);
-
-        init_config ();
-	load_properties ();
+	label = gtk_label_new (g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Score:")));
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 5);
 
+	draw_area = gtk_drawing_area_new ();
 	g_signal_connect (G_OBJECT(draw_area), "button_press_event",
 			  G_CALLBACK (button_press_event), NULL);
+	g_signal_connect (G_OBJECT (draw_area), "configure_event",
+			  G_CALLBACK (configure_event_callback), NULL);
 	g_signal_connect (G_OBJECT (draw_area), "expose_event",
 			  G_CALLBACK (field_expose_event), NULL);
+	gridframe = games_grid_frame_new (FIELDSIZE, FIELDSIZE);
+	gtk_container_add (GTK_CONTAINER (gridframe), draw_area);
+	gtk_table_attach_defaults (GTK_TABLE (table), gridframe, 0, 1, 1, 10);
 
-	g_signal_connect (G_OBJECT (next_draw_area), "expose_event",
-			  G_CALLBACK (preview_expose_event), NULL);
-
-	gtk_widget_realize (next_draw_area); 
+	gtk_widget_set_events (draw_area, gtk_widget_get_events(draw_area) |GDK_BUTTON_PRESS_MASK);
 
 	update_score_state ();
 
 	hbox = gtk_hbox_new (0,3);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, 0, 0, 0);
 
-	/*gnome_app_set_contents (GNOME_APP (app), vbox);*/
+	load_properties ();
 
 	gtk_widget_show_all (app);
 		
