@@ -30,20 +30,12 @@
 #include <games-frame.h>
 
 #include "mahjongg.h"
+#include "drawing.h"
 #include "solubility.h"
-#include "gnome-canvas-pimage.h"
 
 #define APPNAME "mahjongg"
 #define APPNAME_LONG "Mahjongg"
 
-#define AREA_WIDTH 600
-#define AREA_HEIGHT 470
-#define TILE_WIDTH 40
-#define TILE_HEIGHT 56
-#define HALF_WIDTH 18
-#define HALF_HEIGHT 26
-#define THICKNESS 5
-
 /* #defines for the tile selection code. */
 #define SELECTED_FLAG   1
 #define HINT_FLAG       16
@@ -241,36 +233,25 @@ tilepos hard_map [MAX_TILES] = {
 };
 
 tilepos *pos = 0;
-gint xpos_offset;
-gint ypos_offset;
 
 tile tiles[MAX_TILES];
 
 GtkWidget *window, *appbar;
-GtkWidget *canvas;
 GtkWidget *tiles_label;
 gint selected_tile, visible_tiles;
 gint sequence_number;
 
 guint current_seed, next_seed;
 
-static GdkPixbuf *tiles_image, *bg_image;
+gint windowwidth, windowheight;
 
-static gchar *tileset = NULL;
-static gchar *bg_tileset = NULL;
+GList * tileset_list = NULL;
+
+gchar *tileset = NULL;
 static gchar *mapset = NULL;
 static gchar *score_current_mapset = NULL;
 
 static gchar *selected_tileset = NULL;
-static gchar *selected_bg = NULL;
-
-static struct {
-  GdkColor colour ;
-  gchar *name ;
-  gint set;
-} backgnd = {
-	{0, 0, 0, 0}, NULL, 0
-};
 
 struct _maps
 {
@@ -299,6 +280,9 @@ GtkWidget *colour_well = NULL;
 GtkWidget *pref_dialog = NULL;
 GtkWidget *about = NULL;
 
+/* Has the map been changed ? */
+gboolean new_map = TRUE;
+
 typedef enum {
 	NEW_GAME,
 	NEW_GAME_WITH_SEED,
@@ -313,13 +297,10 @@ enum {
 	GAME_DEAD
 } game_over;
 
-static void change_tiles (void);
-static void change_tile_image (tile *tile_inf);
 static void clear_undo_queue (void);
 void you_won (void);
 void no_match (void);
 void check_free (void);
-void load_tiles (gchar *fname, gchar *bg_fname);
 void undo_tile_callback    (GtkWidget *widget, gpointer data);
 void redo_tile_callback    (GtkWidget *widget, gpointer data);
 void hint_callback         (GtkWidget *widget, gpointer data);
@@ -336,7 +317,6 @@ void pause_callback (void);
 void new_game (gboolean with_seed);
 void restart_game (void);
 void select_game (GtkWidget *widget, gpointer data);
-void set_backgnd_colour (gchar *str);
 
 GnomeUIInfo gamemenu [] = {
          GNOMEUIINFO_MENU_NEW_GAME_ITEM(confirm_action, NEW_GAME),
@@ -378,11 +358,6 @@ GnomeUIInfo settingsmenu [] = {
 
 	GNOMEUIINFO_SEPARATOR,
 
-#ifdef SOUND_SUPPORT_FINISHED
-        {GNOME_APP_UI_TOGGLEITEM, N_("_Sound"), NULL, NULL, NULL, NULL,
-        GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-#endif
-
 	GNOMEUIINFO_MENU_PREFERENCES_ITEM(properties_callback, NULL),
 	GNOMEUIINFO_END
 };
@@ -432,12 +407,6 @@ GnomeUIInfo toolbar_uiinfo [] = {
 		shuffle_tiles_callback, NULL, NULL,
 		GNOME_APP_PIXMAP_STOCK, GTK_STOCK_EXECUTE, 0, 0, NULL},
 
-#ifdef SOUND_SUPPORT_FINISHED
-        {GNOME_APP_UI_TOGGLEITEM, N_("Sound"), N_("Toggle sound"),
-		sound_on_callback, NULL, NULL,
-		GNOME_APP_PIXMAP_DATA, mini_sound_xpm, 0, 0, NULL},
-#endif
-
 	{GNOME_APP_UI_ENDOFINFO}
 };
 
@@ -483,9 +452,14 @@ static void set_undoredo_sensitive (gboolean undo, gboolean redo)
 static void
 tileset_callback (GtkWidget *widget, void *data)
 {
+	GList * entry;
+	
+	entry = g_list_nth (tileset_list,
+			    gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));	
+	
 	gconf_client_set_string (conf_client,
 			"/apps/mahjongg/tileset",
-			data,
+			entry->data,
 			NULL);
 }
 
@@ -503,47 +477,12 @@ tileset_changed_cb (GConfClient *client,
 		if (strcmp (tile_tmp, selected_tileset) != 0) {
 			g_free (selected_tileset);
 			selected_tileset = tile_tmp;
-			load_tiles (selected_tileset, selected_bg);
-			change_tiles ();
+			load_images (selected_tileset);
+			draw_all_tiles ();
 		} else {
 			g_free (tile_tmp);
 		}
 	}
-	gnome_canvas_update_now(GNOME_CANVAS(canvas));
-	//FIXME apply in the GUI
-}
-
-static void
-bg_callback (GtkWidget *widget, void *data)
-{
-	gconf_client_set_string (conf_client,
-			"/apps/mahjongg/background",
-			data,
-			NULL);
-}
-
-static void
-bg_changed_cb (GConfClient *client,
-	       guint cnxn_id,
-	       GConfEntry *entry,
-	       gpointer user_data)
-{
-	char *bg_tmp = NULL;
-
-	bg_tmp = gconf_client_get_string (conf_client,
-			"/apps/mahjongg/background", NULL);
-	if (bg_tmp) {
-		if (strcmp (bg_tmp, selected_bg) != 0) {
-			g_free (selected_bg);
-			selected_bg = bg_tmp;
-			load_tiles (selected_tileset, selected_bg);
-			change_tiles ();
-		} else {
-			g_free (bg_tmp);
-		}
-	}
-	gnome_canvas_update_now(GNOME_CANVAS(canvas));
-	//FIXME apply in the GUI
 }
 
 static void
@@ -623,29 +562,28 @@ bg_colour_changed_cb (GConfClient *client,
 		      gpointer user_data)
 {
 	gchar *colour;
-
+	
 	colour = gconf_client_get_string (conf_client,
 			"/apps/mahjongg/bgcolour", NULL);
-	set_backgnd_colour (colour);
+	set_background (colour);
 	if (colour_well != NULL)
 	{
-		gint ur,ug,ub ;
-
-		sscanf (backgnd.name, "#%02x%02x%02x", &ur,&ug,&ub);
-		gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(colour_well),
-				ur, ug, ub, 0);
+		gtk_color_button_set_color (GTK_COLOR_BUTTON(colour_well),
+					    &bgcolour);
 	}
+	draw_all_tiles ();	
 }
 
 static void
 bg_colour_callback (GtkWidget *widget, gpointer data)
 {
 	static char *tmp = "";
-	guint8 r, g, b, a;
+	GdkColor colour;
 
-	gnome_color_picker_get_i8(GNOME_COLOR_PICKER(widget), &r, &g, &b, &a);
+	gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colour);
 
-	tmp = g_strdup_printf ("#%02x%02x%02x", r, g, b);
+	tmp = g_strdup_printf ("#%02x%02x%02x", colour.red,
+			       colour.green, colour.blue);
 
 	gconf_client_set_string (conf_client,
 			"/apps/mahjongg/bgcolour", tmp, NULL);
@@ -668,6 +606,8 @@ mapset_changed_cb (GConfClient *client,
 		g_free (mapset);
 		mapset = mapset_tmp;
 	} 
+
+	new_map = TRUE;
 	
 	dialog = gtk_message_dialog_new (
 		GTK_WINDOW (window),
@@ -684,8 +624,10 @@ mapset_changed_cb (GConfClient *client,
 static void
 set_map_selection (GtkWidget *widget, void *data)
 {
-	struct _maps *map = (struct _maps*) data;
+	struct _maps *map;
 
+	map = &(maps[gtk_combo_box_get_active (GTK_COMBO_BOX (widget))]);
+	
 	g_free (mapset);
 	mapset = g_strdup (map->name);
 
@@ -720,16 +662,6 @@ init_config (void)
 			"/apps/mahjongg/tileset",
 			tileset_changed_cb,
 			NULL, NULL, NULL);
-	gconf_client_notify_add (conf_client,
-			"/apps/mahjongg/background",
-			bg_changed_cb,
-			NULL, NULL, NULL);
-}
-
-static void
-free_str (GtkWidget *widget, void *data)
-{
-	g_free (data);
 }
 
 static void
@@ -777,165 +709,104 @@ update_moves_left (void)
         return moves_left;
 }
 
-void
-set_backgnd_colour (gchar *str)
+static void
+select_tile (gint tileno)
 {
-	GdkColormap *colourmap ;
-	GtkStyle *widget_style, *temp_style;
-
-	g_return_if_fail (str != NULL);
-
-	if (str != backgnd.name) {
-		g_free (backgnd.name) ;
-		backgnd.name = g_strdup (str) ;
-	}
-	colourmap = gtk_widget_get_colormap (canvas);
-	if (!gdk_color_parse (backgnd.name, &backgnd.colour)) {
-		gdk_color_parse ("#34385b", &backgnd.colour);
-	}
-
-	gdk_colormap_alloc_color (colourmap, &backgnd.colour,
-				  FALSE, TRUE);
-
-	widget_style = gtk_widget_get_style (canvas);
-	temp_style = gtk_style_copy (widget_style);
-	temp_style->bg[0] = backgnd.colour;
-	temp_style->bg[1] = backgnd.colour;
-	temp_style->bg[2] = backgnd.colour;
-	temp_style->bg[3] = backgnd.colour;
-	temp_style->bg[4] = backgnd.colour;
-	gtk_widget_set_style (canvas, temp_style);
-	gnome_canvas_update_now (GNOME_CANVAS (canvas));
+        tiles[tileno].selected |= SELECTED_FLAG;
+        draw_tile (tileno);
+        selected_tile = tileno;
 }
 
 static void
-change_tiles (void)
-{
-        gint i;
-        
-        for (i = 0; i < MAX_TILES; i++)
-                change_tile_image (&tiles[i]);
-}
-
-static void
-change_tile_image (tile *tile_inf)
-{
-	gint orig_x, orig_y;
-
-	orig_x = (tile_inf->image % 21) * TILE_WIDTH;
-	orig_y = (tile_inf->image / 21) * TILE_HEIGHT;
-
-	if (tile_inf->selected) {
-		orig_y += 2 * TILE_HEIGHT;
-	}
-
-       /*gdk_pixbuf_finalize (tile_inf->current_image);
-          gdk_pixbuf_finalize (tile_inf->current_bg);*/
-	
-	gdk_pixbuf_copy_area (tiles_image, orig_x, orig_y,
-			      TILE_WIDTH, TILE_HEIGHT,
-			      tile_inf->current_image, 0, 0);
-	gdk_pixbuf_copy_area (bg_image,
-			pos[tile_inf->number].layer * TILE_WIDTH,
-			(tile_inf->selected != 0 ? 1 : 0) * TILE_HEIGHT,
-			TILE_WIDTH, TILE_HEIGHT,
-			tile_inf->current_bg, 0, 0);
-        
-	gnome_canvas_item_set (tile_inf->bg_item, "image",
-			tile_inf->current_bg, NULL);
-	gnome_canvas_item_set (tile_inf->image_item, "image",
-			tile_inf->current_image, NULL);
-}
-
-static void
-select_tile (tile *tile_inf)
-{
-        tile_inf->selected |= SELECTED_FLAG;
-        change_tile_image (tile_inf);
-        selected_tile = tile_inf->number;
-}
-
-static void
-unselect_tile (tile *tile_inf)
+unselect_tile (gint tileno)
 {
         selected_tile = MAX_TILES + 1;
-        tile_inf->selected &= ~SELECTED_FLAG;
-        change_tile_image (tile_inf);
+        tiles[tileno].selected &= ~SELECTED_FLAG;
+        draw_tile (tileno);
 }
 
 static void
-tile_event (GnomeCanvasItem *item, GdkEvent *event, tile *tile_inf)
+remove_pair (gint tile1, gint tile2)
 {
-	char *tmpstr;
+	gchar * tmpstr;
+	
+	tiles[tile1].visible = tiles[tile2].visible = 0;
+	tiles[tile1].selected &= ~SELECTED_FLAG;
+	tiles[tile2].selected &= ~SELECTED_FLAG;
+	draw_tile (tile1);
+	draw_tile (tile2);
+	clear_undo_queue ();
+	tiles[tile1].sequence = tiles[tile2].sequence = sequence_number;
+	sequence_number ++;
+	selected_tile = MAX_TILES + 1;
+	visible_tiles -= 2;
+	tmpstr = g_strdup_printf("%3d", visible_tiles);
+	gtk_label_set_text (GTK_LABEL(tiles_label), tmpstr);
+	g_free (tmpstr);
+	update_moves_left ();
+	set_undoredo_sensitive (TRUE, FALSE);
+	set_menus_sensitive ();
+	
+	if (visible_tiles <= 0) {
+		games_clock_stop(GAMES_CLOCK(chrono));
+		you_won ();
+	}
 
+}
+
+void
+tile_event (gint tileno, gint button)
+{
 	if (paused)
 		return;
 
-	switch (event->type) {
-	case GDK_BUTTON_PRESS :
-		if (tile_free (tile_inf->number)) {
-			switch (event->button.button) {
-			case 1:
-				if (tile_inf->selected & SELECTED_FLAG)
-					unselect_tile (tile_inf);
-				else {
-					if (selected_tile < MAX_TILES) {
-						if ((tiles[selected_tile].type == tile_inf->type) ) {
-							tiles[selected_tile].visible = 0;
-							tile_inf->visible = 0;
-							tiles[selected_tile].selected &= ~SELECTED_FLAG;
-							change_tile_image (&tiles[selected_tile]);
-							gnome_canvas_item_hide (tiles[selected_tile].canvas_item);
-							gnome_canvas_item_hide (tile_inf->canvas_item);
-							clear_undo_queue ();
-							tiles[selected_tile].sequence = tile_inf->sequence = sequence_number;
-							sequence_number ++;
-							selected_tile = MAX_TILES + 1;
-							visible_tiles -= 2;
-							tmpstr = g_strdup_printf("%3d", visible_tiles);
-							gtk_label_set_text (GTK_LABEL(tiles_label), tmpstr);
-							g_free (tmpstr);
-							update_moves_left ();
-							set_undoredo_sensitive (TRUE, FALSE);
-							set_menus_sensitive ();
-
-							if (visible_tiles <= 0) {
-								games_clock_stop(GAMES_CLOCK(chrono));
-								you_won ();
-							}
-						}
-						else
-							no_match ();
-					}
-					else 
-						select_tile (tile_inf);
-				}
-				break;
-                          
-			case 3:
-				if (selected_tile < MAX_TILES) 
-					unselect_tile (&tiles[selected_tile]);
-				select_tile (tile_inf);
-                          
-			default: 
-				break;
-			}
-			break;
-
-		default :
-			break;
+	if (!tile_free (tileno))
+		return;
+	
+	switch (button) {
+	case 1:
+		if (tiles[tileno].selected & SELECTED_FLAG) {
+			unselect_tile (tileno);
+			return;
 		}
+		if (selected_tile >= MAX_TILES) {
+			select_tile (tileno);
+			return;
+		}
+		if ((tiles[selected_tile].type == tiles[tileno].type) ) {
+			remove_pair (selected_tile, tileno);
+			return;
+		}
+		no_match ();
+	break;
+	
+	case 3:
+		if (selected_tile < MAX_TILES) 
+			unselect_tile (selected_tile);
+		select_tile (tileno);
+		
+	default: 
+		break;
 	}
 }
 
 static void
-fill_tile_menu (GtkWidget *menu, gchar *sdir, gint is_tile)
+fill_tile_menu (GtkWidget *menu, gchar *sdir)
 {
 	struct dirent *e;
 	DIR *dir;
         gint itemno = 0;
 	gchar *dname = NULL;
 
+	if (tileset_list) {
+		g_list_foreach (tileset_list,
+				(GFunc) g_free,
+				NULL);
+		g_list_free (tileset_list);
+	}
+
+	tileset_list = NULL;
+	
 	dname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
 			(sdir), FALSE, NULL);
 	dir = opendir (dname);
@@ -946,43 +817,24 @@ fill_tile_menu (GtkWidget *menu, gchar *sdir, gint is_tile)
 	}
 
 	while ((e = readdir (dir)) != NULL){
-		GtkWidget *item;
 		gchar *s = g_strdup (e->d_name);
 
-		if (!(strstr (e->d_name, ".xpm") ||
-		      strstr (e->d_name, ".gif") ||
-		      strstr (e->d_name, ".png") ||
-		      strstr (e->d_name, ".jpg") ||
-		      strstr (e->d_name, ".xbm"))){
-			free (s);
+		if (!(g_strrstr (s, ".xpm") ||
+		      g_strrstr (s, ".svg") ||
+		      g_strrstr (s, ".gif") ||
+		      g_strrstr (s, ".png") ||
+		      g_strrstr (s, ".jpg") ||
+		      g_strrstr (s, ".xbm"))){
+			g_free (s);
 			continue;
 		}
 
-		item = gtk_menu_item_new_with_label (s);
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);
-                if (is_tile) {
-                        g_signal_connect (G_OBJECT (item), "activate",
-                                          G_CALLBACK (tileset_callback), s); 
-                        g_signal_connect (G_OBJECT (item), "destroy",
-                                          G_CALLBACK (free_str), s);
-                } else {
-                        g_signal_connect (G_OBJECT (item), "activate",
-                                          G_CALLBACK (bg_callback), s); 
-                        g_signal_connect (G_OBJECT (item), "destroy",
-                                          G_CALLBACK (free_str), s);
-                }
-
-		if (is_tile) {
-			if (!strcmp(tileset, s)) {
-				gtk_menu_set_active(GTK_MENU(menu), itemno);
-			}
-		} else {
-			if (!strcmp(bg_tileset, s)) {
-				gtk_menu_set_active(GTK_MENU(menu), itemno);
-			}
+		gtk_combo_box_append_text (GTK_COMBO_BOX (menu), s);
+		tileset_list = g_list_append (tileset_list, s);
+		if (!strcmp(tileset, s)) {
+			gtk_combo_box_set_active(GTK_COMBO_BOX (menu), itemno);
 		}
-
+		
 	        itemno++;
 	}
 
@@ -994,20 +846,13 @@ static void
 fill_map_menu (GtkWidget *menu)
 {
 	gint lp, itemno=0 ;
-	GtkWidget *item;
 
 	for (lp=0;lp<G_N_ELEMENTS(maps);lp++) {
-		gchar *str = g_strdup (_(maps[lp].name)) ;
+		gchar *str = g_strdup(_(maps[lp].name)) ;
 
-		item = gtk_menu_item_new_with_label (str) ;
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_signal_connect (G_OBJECT(item), "activate",
-				  G_CALLBACK (set_map_selection), &maps[lp]); 
-		g_signal_connect (G_OBJECT(item), "destroy",
-				  G_CALLBACK (free_str), str); 
+		gtk_combo_box_append_text (GTK_COMBO_BOX (menu), str) ;
 		if (!g_ascii_strcasecmp (mapset, maps[lp].name))
-			gtk_menu_set_active (GTK_MENU (menu), itemno); 
+			gtk_combo_box_set_active (GTK_COMBO_BOX (menu), itemno); 
 		itemno++ ;
 	}
 }
@@ -1059,10 +904,19 @@ check_free (void)
 						     GTK_DIALOG_MODAL
 						     | GTK_DIALOG_DESTROY_WITH_PARENT,
 						     GTK_MESSAGE_INFO,
-						     GTK_BUTTONS_OK,
+						     GTK_BUTTONS_NONE,
 						     (_("There are no more moves.")));
+			gtk_dialog_add_buttons (GTK_DIALOG (mb),
+						_("Shuffle"),
+						GTK_RESPONSE_REJECT,
+						GTK_STOCK_OK,
+						GTK_RESPONSE_ACCEPT,
+						NULL);
+			gtk_dialog_set_default_response (GTK_DIALOG (mb),
+							 GTK_RESPONSE_ACCEPT);
 			gtk_dialog_set_has_separator (GTK_DIALOG (mb), FALSE);
-			gtk_dialog_run (GTK_DIALOG (mb));
+			if (gtk_dialog_run (GTK_DIALOG (mb)) == GTK_RESPONSE_REJECT)
+				shuffle_tiles_callback (NULL, NULL);
 			gtk_widget_destroy (mb);
                 }
                 game_over = GAME_LOST;
@@ -1108,7 +962,7 @@ pref_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 void
 properties_callback (GtkWidget *widget, gpointer data)
 {
-	GtkWidget *menu, *omenu;
+	GtkWidget *omenu;
 	GtkWidget *frame, *table, *w, *label;
 	GtkSizeGroup *group;
 	GtkWidget *top_table;
@@ -1150,23 +1004,12 @@ properties_callback (GtkWidget *widget, gpointer data)
 	gtk_size_group_add_widget (group, label);
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
 
-	omenu = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-	fill_tile_menu (menu, "mahjongg", 1);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
+	omenu = gtk_combo_box_new_text ();
+	fill_tile_menu (omenu, "mahjongg");
+	g_signal_connect (G_OBJECT (omenu), "changed",
+			  G_CALLBACK (tileset_callback), NULL); 
 	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
 
-	label = gtk_label_new (_("Tile Background"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_size_group_add_widget (group, label);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
-
-	omenu = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-	fill_tile_menu (menu, "mahjongg/bg", 0);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
-	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 1, 2);
-
 	gtk_container_add(GTK_CONTAINER (frame), table);
 
 	frame = games_frame_new (_("Maps"));
@@ -1182,10 +1025,10 @@ properties_callback (GtkWidget *widget, gpointer data)
 	gtk_size_group_add_widget (group, label);
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
 
-	omenu = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-	fill_map_menu (menu);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
+	omenu = gtk_combo_box_new_text ();
+	fill_map_menu (omenu);
+	g_signal_connect (G_OBJECT (omenu), "changed",
+			  G_CALLBACK (set_map_selection), NULL); 
 	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
 
 	gtk_container_add(GTK_CONTAINER (frame), table);
@@ -1203,15 +1046,10 @@ properties_callback (GtkWidget *widget, gpointer data)
 	gtk_size_group_add_widget (group, label);
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
 
-	{
-	  int ur, ug, ub;
-
-	  w  = gnome_color_picker_new ();
-	  sscanf (backgnd.name, "#%02x%02x%02x", &ur, &ug, &ub);
-	  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (w), ur, ug, ub, 0);
-	  g_signal_connect (G_OBJECT (w), "color_set",
-			    G_CALLBACK (bg_colour_callback), &backgnd.name);
-	}
+	w  = gtk_color_button_new ();
+	gtk_color_button_set_color (GTK_COLOR_BUTTON (w), &bgcolour);
+	g_signal_connect (G_OBJECT (w), "color_set",
+			  G_CALLBACK (bg_colour_callback), NULL);
 	gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 0, 1);
 
 	gtk_container_add (GTK_CONTAINER (frame), table);
@@ -1249,9 +1087,9 @@ hint_timeout (gpointer data)
 
 	tiles[hint_tiles[0]].selected ^= HINT_FLAG;
 	tiles[hint_tiles[1]].selected ^= HINT_FLAG;
-	change_tile_image(&tiles[hint_tiles[0]]);
-	change_tile_image(&tiles[hint_tiles[1]]);
-
+	draw_tile (hint_tiles[0]);
+	draw_tile (hint_tiles[1]);
+	
 	return 1;
 }
 
@@ -1276,7 +1114,7 @@ hint_callback (GtkWidget *widget, gpointer data)
 	/* Clear any selection */
 	if (selected_tile < MAX_TILES) {
 		tiles[selected_tile].selected = 0;
-		change_tile_image (&tiles[selected_tile]);
+		draw_tile (selected_tile);
 		selected_tile = MAX_TILES + 1;
 	}
                 
@@ -1288,8 +1126,8 @@ hint_callback (GtkWidget *widget, gpointer data)
 				if (free) {
 					tiles[i].selected ^= HINT_FLAG;
 					tiles[j].selected ^= HINT_FLAG;
-					change_tile_image (&tiles[i]);
-					change_tile_image (&tiles[j]);
+					draw_tile (i);
+					draw_tile (j);
 					hint_tiles[0] = i;
 					hint_tiles[1] = j;
 				}
@@ -1302,7 +1140,7 @@ hint_callback (GtkWidget *widget, gpointer data)
 	   tiles[i].visible = 0 ;*/
                 
 	timeout_counter = 0;
-	timer = gtk_timeout_add (250, (GtkFunction) hint_timeout, NULL);
+	timer = g_timeout_add (250, (GSourceFunc) hint_timeout, NULL);
                 
 	/* 30s penalty */
 	games_clock_stop (GAMES_CLOCK(chrono));
@@ -1328,6 +1166,7 @@ about_callback (GtkWidget *widget, gpointer data)
 		"Heinz Hempe",
 		"Michael Meeks",
                 "Philippe Chavin",
+		"Callum McKenzie",
 		"",
 		N_("Tiles:"),
 		"Jonathan Buzzard",
@@ -1381,19 +1220,13 @@ about_callback (GtkWidget *widget, gpointer data)
 void
 pause_callback (void)
 {
-        gint i;
         paused = !paused;
+	draw_all_tiles ();
         if (paused) {
                 games_clock_stop (GAMES_CLOCK (chrono));
-                for (i = 0; i < MAX_TILES; i++)
-                        if (tiles[i].visible)
-                                gnome_canvas_item_hide (tiles[i].image_item);
                 message(_("... Game paused ..."));
         }
         else {
-                for (i = 0; i < MAX_TILES; i++)
-                        if (tiles[i].visible)
-                                gnome_canvas_item_show (tiles[i].image_item);
                 message ("");
                 games_clock_start (GAMES_CLOCK(chrono));
         }
@@ -1401,13 +1234,8 @@ pause_callback (void)
 
 void ensure_pause_off (void)
 {
-        gint i;
-
         if (paused) {
-                gtk_toggle_button_set_active (PAUSE_BUTTON, FALSE);
-                for (i = 0; i < MAX_TILES; i++)
-                        if (tiles[i].visible)
-                                gnome_canvas_item_show (tiles[i].image_item);
+		draw_all_tiles ();
                 message("");
         }
         paused = FALSE;
@@ -1435,7 +1263,6 @@ init_game (void)
         sequence_number = 1 ;
         visible_tiles = MAX_TILES;
         selected_tile = MAX_TILES + 1;
-        gnome_canvas_update_now(GNOME_CANVAS(canvas));
 	set_undoredo_sensitive (FALSE, FALSE);
 	set_menus_sensitive ();
 
@@ -1479,11 +1306,8 @@ restart_game (void)
         tiles[i].visible = 1;
         tiles[i].selected = 0;
 	tiles[i].sequence = 0;
-	if (i == selected_tile)
-	  change_tile_image (&tiles[selected_tile]);
-	  
-	gnome_canvas_item_show (tiles[i].canvas_item);
     }
+    draw_all_tiles ();
     init_game ();
 }
 
@@ -1500,7 +1324,7 @@ redo_tile_callback (GtkWidget *widget, gpointer data)
         
         if (selected_tile<MAX_TILES) {
                 tiles[selected_tile].selected = 0 ;
-                change_tile_image (&tiles[selected_tile]);
+                draw_tile(selected_tile);
                 selected_tile = MAX_TILES + 1; 
         }
         change = 0 ;
@@ -1508,7 +1332,7 @@ redo_tile_callback (GtkWidget *widget, gpointer data)
                 if (tiles[i].sequence == sequence_number) {
                         tiles[i].selected = 0 ;
                         tiles[i].visible = 0 ;
-                        gnome_canvas_item_hide (tiles[i].canvas_item);
+			draw_tile (i);
                         visible_tiles-- ;
                         change = 1 ;
                 }
@@ -1528,7 +1352,6 @@ redo_tile_callback (GtkWidget *widget, gpointer data)
 	}
 	
         update_moves_left ();
-        gnome_canvas_update_now (GNOME_CANVAS (canvas));
 }
 
 void
@@ -1548,7 +1371,7 @@ undo_tile_callback (GtkWidget *widget, gpointer data)
         
         if (selected_tile<MAX_TILES) {
                 tiles[selected_tile].selected = 0 ;
-                change_tile_image (&tiles[selected_tile]);
+                draw_tile (selected_tile);
                 selected_tile = MAX_TILES + 1; 
         }
         
@@ -1557,13 +1380,12 @@ undo_tile_callback (GtkWidget *widget, gpointer data)
                         tiles[i].selected = 0 ;
                         tiles[i].visible = 1 ;
                         visible_tiles++ ;
-                        gnome_canvas_item_show (tiles[i].canvas_item);
+			draw_tile (i);
                 }
 
         tmpstr = g_strdup_printf ("%3d", visible_tiles);
         gtk_label_set_text (GTK_LABEL(tiles_label), tmpstr);
 	g_free (tmpstr);
-        gnome_canvas_update_now (GNOME_CANVAS (canvas));
 
 	set_menus_sensitive ();
 	set_undoredo_sensitive (sequence_number>1, TRUE);
@@ -1639,10 +1461,10 @@ load_map (void)
 {
 	gchar* name = mapset;
 	gint lp ;
-	gint xmax = 0, ymax = 0;
-	tilepos *t;
 	gboolean found;
 
+	new_map = FALSE;
+		
 	found = FALSE;
 	for (lp=0;lp<G_N_ELEMENTS(maps);lp++)
 		if (g_ascii_strcasecmp (maps[lp].name, name) == 0) {
@@ -1660,167 +1482,9 @@ load_map (void)
 
 	pos = maps[lp].map ;
 
-	for (t = pos ; t < pos + MAX_TILES ; t++) {
-		if ( (*t).x  > xmax )
-			xmax = (*t).x;
-		if ( (*t).y  > ymax )
-			ymax = (*t).y;
-	}
-	xpos_offset = ( AREA_WIDTH - (HALF_WIDTH * (xmax+1)) ) / 2;
-	ypos_offset = ( AREA_HEIGHT - (HALF_HEIGHT * (ymax+1) ) ) / 2;
-	
-	generate_dependancies() ;
-}
-
-static gint
-canvas_x (gint i)
-{
-	return pos[i].x * (HALF_WIDTH-0) + xpos_offset
-		+ (THICKNESS * pos[i].layer);
-}
-
-static gint
-canvas_y (gint i)
-{
-	return pos[i].y * (HALF_HEIGHT-0) + ypos_offset
-		- (THICKNESS * pos[i].layer);
-}
-
-static void
-load_images (void)
-{
-	gint i;
-  
-	for (i = MAX_TILES - 1; i >= 0; i --) {
-		gnome_canvas_item_set (tiles[i].image_item,
-				       "x", (double)canvas_x (i),
-				       "y", (double)canvas_y (i),
-				       NULL);
-		gnome_canvas_item_set (tiles[i].bg_item,
-				       "x", (double)canvas_x (i),
-				       "y", (double)canvas_y (i),
-				       NULL);
-	}
-}
-
-static void
-create_canvas_items (void)
-{
-	gint orig_x, orig_y, i;
-  
-	/* It's essential that the tiles are already sorted ginto layer order (lowest first) */
-	for (i = MAX_TILES - 1; i >= 0; i --) {
-		tiles[i].canvas_item = gnome_canvas_item_new (gnome_canvas_root(GNOME_CANVAS(canvas)),
-							      gnome_canvas_group_get_type (),
-							      NULL);
-		orig_x = (tiles[i].image % 21) * TILE_WIDTH;
-		orig_y = (tiles[i].image / 21) * TILE_HEIGHT;
-	
-		tiles[i].number = i;
-
-		tiles[i].current_image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace(tiles_image),
-							 TRUE, gdk_pixbuf_get_bits_per_sample(tiles_image),
-							 TILE_WIDTH, TILE_HEIGHT);
-		tiles[i].current_bg = gdk_pixbuf_new (gdk_pixbuf_get_colorspace(tiles_image),
-						      TRUE, gdk_pixbuf_get_bits_per_sample(tiles_image),
-						      TILE_WIDTH, TILE_HEIGHT);
-	
-		gdk_pixbuf_copy_area (tiles_image, orig_x, orig_y,
-				      TILE_WIDTH, TILE_HEIGHT,
-				      tiles[i].current_image, 0, 0);
-		gdk_pixbuf_copy_area (bg_image, pos[i].layer * TILE_WIDTH, 0,
-				      TILE_WIDTH, TILE_HEIGHT,
-				      tiles[i].current_bg, 0, 0);
-
-
-		tiles[i].bg_item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (tiles[i].canvas_item),
-							  gnome_canvas_pimage_get_type(),
-							  "image", tiles[i].current_bg,
-							  "x", (double)canvas_x(i),
-							  "y", (double)canvas_y(i),
-							  "width", (double)TILE_WIDTH,
-							  "height", (double)TILE_HEIGHT,
-							  NULL);
-
-		tiles[i].image_item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (tiles[i].canvas_item),
-							     gnome_canvas_pimage_get_type(),
-							     "image", tiles[i].current_image,
-							     "x", (double)canvas_x(i),
-							     "y", (double)canvas_y(i),
-							     "width", (double)TILE_WIDTH,
-							     "height", (double)TILE_HEIGHT,
-							     NULL);
-	
-		g_signal_connect (G_OBJECT (tiles[i].canvas_item), "event",
-				  G_CALLBACK (tile_event), &tiles[i]);
-	}
-}
-
-void
-load_tiles (gchar *fname, gchar *bg_fname)
-{
-	gchar *tmp, *fn, *bg_fn;
-
-	tmp = g_build_filename ("mahjongg", fname, NULL);
-	
-	fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					(tmp), FALSE, NULL);
-	g_free (tmp);
-
-	tmp = g_build_filename ("mahjongg", "bg", bg_fname, NULL);
-
-	bg_fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					   (tmp), FALSE, NULL);
-	g_free (tmp);
-
-	if (!g_file_test ((fn), G_FILE_TEST_EXISTS)) {
-		gchar *s = g_strdup_printf (_("Could not find file %s"), fn);
-		GtkWidget *box;
-
-		box = gtk_message_dialog_new (GTK_WINDOW (window),
-					      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-					      GTK_MESSAGE_ERROR,
-					      GTK_BUTTONS_OK,
-					      s);
-		gtk_dialog_set_has_separator (GTK_DIALOG (box), FALSE);
-		gtk_dialog_run (GTK_DIALOG (box));
-		
-		exit (1);
-	}
-
-	if (!g_file_test ((bg_fn), G_FILE_TEST_EXISTS)) {
-		gchar *s = g_strdup_printf (_("Could not find file %s"), bg_fn);
-		GtkWidget *box;
-	
-		box = gtk_message_dialog_new (GTK_WINDOW (window),	
-					      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-					      GTK_MESSAGE_ERROR,
-					      GTK_BUTTONS_OK,
-					      s);
-		gtk_dialog_set_has_separator (GTK_DIALOG (box), FALSE);
-		gtk_dialog_run (GTK_DIALOG (box));
-
-		exit (1);
-	}
-
-	g_free (tileset);
-	tileset = g_strdup(fname);
-	
-	g_free (bg_tileset);
-	bg_tileset = g_strdup(bg_fname);
-	
-	if (tiles_image)
-		gdk_pixbuf_unref (tiles_image);
-
-	if (bg_image)
-		gdk_pixbuf_unref (bg_image);
-
-	tiles_image = gdk_pixbuf_new_from_file (fn, NULL);
-
-        bg_image = gdk_pixbuf_new_from_file (bg_fn, NULL);
-
-	g_free (bg_fn);
-	g_free (fn);
+	generate_dependencies ();
+	calculate_view_geometry ();
+	configure_pixmaps (); 
 }
 
 static void
@@ -1832,61 +1496,59 @@ do_game (void)
 	str = g_strdup_printf ("%s (%d)",_("Mahjongg"), current_seed);
 	gtk_window_set_title (GTK_WINDOW (window), str);
 	g_free (str);
-	
-	load_map (); /* assigns pos, and calculates dependencies */
+
+	if (new_map)
+		load_map ();
 	generate_game (current_seed); /* puts in the positions of the tiles */
 }
 
-static void
-create_mahjongg_board (GtkWidget *mbox)
+/* Record any changes to our window size. */
+static gboolean window_configure_cb (GtkWidget *w, GdkEventConfigure *e,
+				 gpointer data)
 {
-	gchar *buf, *buf2;
+	gconf_client_set_int (conf_client, "/apps/mahjongg/width",
+			      e->width, NULL);
+	gconf_client_set_int (conf_client, "/apps/mahjongg/height",
+			      e->height, NULL);
+
+	return FALSE;
+}
+
+static void
+load_preferences (void)
+{
+	gchar *buf;
 	
-	canvas = gnome_canvas_new();
-
-	gtk_box_pack_start_defaults (GTK_BOX (mbox), canvas);
-
-	gtk_widget_set_size_request (GTK_WIDGET (canvas),
-				     AREA_WIDTH, AREA_HEIGHT);
-	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (canvas), 1);
-	gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas),
-					0, 0, AREA_WIDTH, AREA_HEIGHT);
-	gtk_widget_show (canvas);
-
 	mapset = gconf_client_get_string (conf_client,
 			"/apps/mahjongg/mapset", NULL);
 
 	buf = gconf_client_get_string (conf_client,
 			"/apps/mahjongg/bgcolour", NULL) ;
-	set_backgnd_colour (buf) ;
+	set_background (buf) ;
 	g_free (buf);
 
 	buf = gconf_client_get_string (conf_client, "/apps/mahjongg/tileset", NULL);
 	if (buf == NULL) {
-		buf = g_strdup("default.png");
-	} else {
-		selected_tileset = g_strdup (buf);
+		buf = g_strdup("default.svg");
 	}
+	selected_tileset = g_strdup (buf);
+	g_free (buf);	
 	
-	buf2 = gconf_client_get_string (conf_client, "/apps/mahjongg/background", NULL);
-	if (buf2 == NULL) {
-		buf2 = g_strdup("bg1.png");
-	} else {
-		selected_bg = g_strdup (buf2);
-	}
-        
 	popup_warn = gconf_client_get_bool (conf_client,
 					    "/apps/mahjongg/warn", NULL);
+
+	windowwidth = gconf_client_get_int (conf_client,
+					    "/apps/mahjongg/width", NULL);
+	if (windowwidth <= 0)
+		windowwidth = 530;
 	
-	do_game ();
-	load_tiles (selected_tileset, selected_bg);
+	windowheight = gconf_client_get_int (conf_client,
+					     "/apps/mahjongg/height", NULL);
+	if (windowheight <= 0)
+		windowheight = 440;
 
-	create_canvas_items ();
-
-	g_free (buf2);
-	g_free (buf);
-	init_game ();
-	update_score_state ();
+	
+	load_images (selected_tileset);
 }
 
 static void
@@ -1901,17 +1563,12 @@ new_seed ()
 void
 new_game (gboolean re_seed)
 {
-	gint i;
-
 	if (re_seed)
 		new_seed ();
-	do_game ();
-	load_images ();
 
-	for (i = 0; i < MAX_TILES; i++) {
-		change_tile_image (&tiles[i]);
-		gnome_canvas_item_show (tiles[i].canvas_item);
-	}
+	do_game ();
+
+	draw_all_tiles ();
 
 	init_game ();
 
@@ -1925,33 +1582,14 @@ new_game (gboolean re_seed)
 void
 shuffle_tiles_callback (GtkWidget *widget, gpointer data)
 {
-        gint i, previous = 0, first=1, num_shuffle=0;
-        tile temp;
         time_t seconds;
+	gboolean ok;
 
         if (paused || game_over == GAME_DEAD || game_over == GAME_WON) return;
 
-        do {
-                num_shuffle++;
-                /* We do a circular permutation */
-                for (i=0; i<MAX_TILES; i++) {
-                        if (tiles[i].visible) {
-                                if (first) {
-                                        temp = tiles[i];
-                                        first--; }
-                                else {
-                                        tiles[previous].type = tiles[i].type;
-                                        tiles[previous].image = tiles[i].image;
-                                }
-                                previous = i; 
-                        }
-                }
-                tiles[previous].type = temp.type;
-                tiles[previous].image = temp.image;
-        }
-        while (!(update_moves_left ()) && num_shuffle < visible_tiles);
-        
-        if (num_shuffle >= visible_tiles) {
+	ok = shuffle ();
+   
+	if (!ok) {
                 GtkWidget *mb;
                 game_over = GAME_DEAD;
                 games_clock_stop (GAMES_CLOCK (chrono));
@@ -1965,14 +1603,8 @@ shuffle_tiles_callback (GtkWidget *widget, gpointer data)
 		gtk_widget_destroy (mb);
 
         } else {
-                
-                for (i=0; i<MAX_TILES; i++) {
-                        tiles[i].sequence = 0;
-                        if (tiles[i].visible) {
-                                change_tile_image (&tiles[i]);
-                                gnome_canvas_item_show (tiles[i].canvas_item);
-                        }
-                }
+
+                draw_all_tiles ();
                 
                 game_over = GAME_RUNNING;
 
@@ -1981,6 +1613,11 @@ shuffle_tiles_callback (GtkWidget *widget, gpointer data)
                 seconds = GAMES_CLOCK(chrono)->stopped;
                 games_clock_set_seconds(GAMES_CLOCK(chrono), (gint) (seconds+60));
                 games_clock_start (GAMES_CLOCK(chrono));
+
+		/* Disable undo/redo after a shuffle. */
+		sequence_number = 1;
+		clear_undo_queue ();
+		set_undoredo_sensitive (FALSE, FALSE);
         }
 	
 	set_menus_sensitive ();
@@ -1989,7 +1626,7 @@ shuffle_tiles_callback (GtkWidget *widget, gpointer data)
 int
 main (int argc, char *argv [])
 {
-	GtkWidget *mbox;
+	GtkWidget *board;
 	GtkWidget *chrono_label;
 	GtkWidget *status_box;
 	GtkWidget *group_box;
@@ -2014,12 +1651,16 @@ main (int argc, char *argv [])
 	if (!games_gconf_sanity_check_string (conf_client, "/apps/mahjongg/tileset")) {
 		return 1;
 	}
-
+	load_preferences ();
+	
 	new_seed ();
 
 	window = gnome_app_new (APPNAME, _(APPNAME_LONG));
-	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
-
+	gtk_window_set_default_size (GTK_WINDOW (window), windowwidth,
+				     windowheight);
+	g_signal_connect (G_OBJECT (window), "configure_event",
+			  G_CALLBACK (window_configure_cb), NULL);
+	
 	/* Statusbar for a chrono, Tiles left and Moves left */
 	status_box = gtk_hbox_new (FALSE, 10);
 
@@ -2065,10 +1706,19 @@ main (int argc, char *argv [])
 	g_signal_connect (G_OBJECT (window), "delete_event",
 			  G_CALLBACK (delete_event_callback), (gpointer)QUIT_GAME);
 
-	mbox = gtk_vbox_new (FALSE, 0);
-	gnome_app_set_contents (GNOME_APP (window), mbox);
-	create_mahjongg_board (mbox);
+	board = create_mahjongg_board ();
+	gnome_app_set_contents (GNOME_APP (window), board);
 
+	/* FIXME: get these in the best place (as per the comment below. */
+	init_config ();
+
+	do_game ();
+	init_game ();
+	score_current_mapset = g_strdup (mapset);
+	update_score_state ();
+	
+	/* Note: we have to have a layout loaded before here so that the
+	 * window knows how big to make the tiles. */
 	gtk_widget_show (window);
 
 	show = gconf_client_get_bool (conf_client,
@@ -2088,11 +1738,6 @@ main (int argc, char *argv [])
 		gtk_widget_queue_resize (window);
 	}
 
-	init_config ();
-
-	score_current_mapset = g_strdup (mapset);
-	update_score_state ();
-
   	gnome_app_flash (GNOME_APP (window), 
   				_("Welcome to GNOME Mahjongg!")); 
 
